using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;

namespace Angular_Velocity
{
    public class Sprite
    {
        private ContentManager p_content;
        private SpriteBatch p_spriteBatch;
        public Texture2D image;
        public Vector2 position;
        public Vector2 velocityLinear;
        public Color color;
        public float rotation;
        public float velocityAngular;
        public Vector2 scaleV;
        public Vector2 origin;
        public bool alive;
        public bool visible;

        public Sprite(ContentManager content, SpriteBatch spriteBatch)
        {
            p_content = content;
            p_spriteBatch = spriteBatch;
            image = null;
            position = Vector2.Zero;
            velocityLinear = Vector2.Zero;
            color = Color.White;
            rotation = 0.0f;
            velocityAngular = 0.0f;
            scaleV = new Vector2(1.0f);
            origin = Vector2.Zero;
            alive = true;
            visible = true;
        }

        public float scale
        {
            get { return scaleV.X; }
            set
            {
                scaleV.X = value;
                scaleV.Y = value;
            }
        }

        public bool Load(string assetName)
        {
            try
            {
                image = p_content.Load<Texture2D>(assetName);
                origin = new Vector2(image.Width / 2, image.Height / 2);
            }
            catch (Exception) { return false; }
            return true;
        }

        public void Draw()
        {
            p_spriteBatch.Draw(image, position, null, color, rotation,
                origin, scaleV, SpriteEffects.None, 0.0f);
        }

        public void Move()
        {
            position += velocityLinear;
        }

        public void Rotate()
        {
            rotation += velocityAngular;
            if (rotation > Math.PI * 2)
                rotation -= (float)Math.PI * 2;
            else if (rotation < 0.0f)
                rotation = (float)Math.PI * 2 - rotation;
        }
    } 

    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Viewport screen;
        SpriteFont font;
        Sprite shuttle, planet;
        float orbitRadius, orbitAngle;
        Vector2 oldPos;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            TargetElapsedTime = TimeSpan.FromTicks(333333);
        }

        protected override void Initialize()
        {
            base.Initialize();
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);
            screen = GraphicsDevice.Viewport;

            //create the font
            font = Content.Load<SpriteFont>("WascoSans");

            //create the planet sprite
            planet = new Sprite(Content, spriteBatch);
            planet.Load("planet1");
            planet.scale = 0.5f;
            planet.position = new Vector2(400, 240);

            //create the ship sprite
            shuttle = new Sprite(Content, spriteBatch);
            shuttle.Load("shuttle");
            shuttle.scale = 0.2f;

            orbitRadius = 200.0f;
            orbitAngle = 0.0f;
        }

        protected override void Update(GameTime gameTime)
        {
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            //remember position for orientation
            oldPos = shuttle.position;

            //keep angle within 0-360 degs
            orbitAngle += 1.0f;
            if (orbitAngle > 360.0f)
                orbitAngle -= 360.0f;

            //calculate shuttle position
            float x = 400 + (float)Math.Cos(MathHelper.ToRadians(orbitAngle)) 
                * orbitRadius;
            float y = 240 + (float)Math.Sin(MathHelper.ToRadians(orbitAngle)) 
                * orbitRadius;
            
            //move the position
            shuttle.position = new Vector2(x, y);

            //point shuttle's nose in the right direction
            float angle = TargetAngle(shuttle.position, oldPos);

            //subtract 180 degrees to reverse the direction
            angle = MathHelper.WrapAngle(angle - MathHelper.ToRadians(180));

            //adjust for artwork pointing up
            angle += MathHelper.ToRadians(90);

            //update shuttle's rotation
            shuttle.rotation = angle;


            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.Black);
            spriteBatch.Begin();

            planet.Draw();
            shuttle.Draw();

            string text = "Angle: " + shuttle.rotation.ToString("N4");
            //spriteBatch.DrawString(font, text, new Vector2(200, 440), Color.White);

            text = "Radius: " + orbitRadius.ToString("N0");
            spriteBatch.DrawString(font, text, new Vector2(450, 440), Color.White);

            spriteBatch.End();
            base.Draw(gameTime);
        }


        float TargetAngle(Vector2 p1, Vector2 p2)
        {
            return TargetAngle(p1.X, p1.Y, p2.X, p2.Y);
        }

        float TargetAngle(double x1,double y1,double x2,double y2)
        {
            double deltaX = (x2-x1);
            double deltaY = (y2-y1);
            return (float)Math.Atan2(deltaY,deltaX);
        }

    }
}
