using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;

namespace GameLibrary
{
    /*
     * GUI base control class
     */
    public abstract class Control : Sprite 
    {
        protected SpriteFont p_font;

        public Control(ContentManager content, SpriteBatch spriteBatch, SpriteFont font)
            : base(content, spriteBatch)
        {
            p_font = font;
        }

        public override bool Load(string filename)
        {
            return base.Load(filename);
        }

        public virtual void Update(TouchLocation touch)
        {
        }

        public override void Draw()
        {
            base.Draw();
        }
    }

    /*
     * GUI Label control
     */
    public class Label : Control
    {
        public string text;
        public Color shadowColor;
        public Color textColor;
        public bool UseShadow;

        public Label(ContentManager content, SpriteBatch spriteBatch, SpriteFont font)
            : base(content, spriteBatch, font)
        {
            text = "";
            color = Color.White;
            textColor = Color.White;
            shadowColor = Color.Black;
            UseShadow = true;
        }

        public override void Update(TouchLocation touch)
        {
            base.Update(touch);
        }

        public override void Draw()
        {
            if (UseShadow)
            {
                p_spriteBatch.DrawString(p_font, text,
                    new Vector2(position.X - 2, position.Y - 2), shadowColor);
            }
            p_spriteBatch.DrawString(p_font, text, position, textColor);
        }

        public Vector2 TextSize()
        {
            return p_font.MeasureString(text);
        }
    }

    /*
     * GUI Button control
     */
    public class Button : Control
    {
        public string text;
        public Color shadowColor;
        public Color textColor;
        public bool UseShadow;
        public bool Tapped;

        public Button(ContentManager content, SpriteBatch spriteBatch, SpriteFont font)
            : base(content, spriteBatch, font)
        {
            text = "";
            color = Color.White;
            textColor = Color.White;
            shadowColor = Color.Black;
            UseShadow = true;
            Load("button");
        }

        public override void Update(TouchLocation touch)
        {
            base.Update(touch);
            Tapped = false;
            if (touch.State == TouchLocationState.Pressed)
            {
                Rectangle rect = Boundary();
                Vector2 pos = touch.Position;
                Point point = new Point((int)pos.X, (int)pos.Y);
                if (rect.Contains(point))
                {
                    Tapped = true;
                }
            }
        }

        public override void Draw()
        {
            base.Draw();
            Vector2 size = TextSize();
            Vector2 pos2 = new Vector2(position.X + 2, position.Y + 2);
            Vector2 pivot = new Vector2(size.X / 2, size.Y / 2);
            p_spriteBatch.DrawString(p_font, text, position, shadowColor, 0.0f, pivot, 1.0f, SpriteEffects.None, zindex);
            p_spriteBatch.DrawString(p_font, text, pos2, textColor, 0.0f, pivot, 1.0f, SpriteEffects.None, zindex);
        }

        public Vector2 TextSize()
        {
            return p_font.MeasureString(text);
        }
    }

    /*
     * GUI Horizonal Slider control
     */
    public class HSlider : Control
    {
        public bool Moving;
        public Vector2 start;
        private int p_value;
        private int p_limit;
        Sprite sprLeftEnd, sprRightEnd, sprBar;

        public HSlider(ContentManager content, SpriteBatch spriteBatch, SpriteFont font)
            : base(content, spriteBatch, font)
        {
            scale = 1.0f;
            start = Vector2.Zero;

            Load("slider_tab");

            sprLeftEnd = new Sprite(content, spriteBatch);
            sprLeftEnd.Load("hslider_end");
            sprLeftEnd.origin = new Vector2(3, 16);

            sprRightEnd = new Sprite(content, spriteBatch);
            sprRightEnd.Load("hslider_end");
            sprRightEnd.origin = new Vector2(0, 16);

            sprBar = new Sprite(content, spriteBatch);
            sprBar.Load("hslider_bar");
            sprBar.origin = new Vector2(0, 16);
            Limit = 100;
        }

        public int Value
        {
            get { return p_value; }
            set 
            { 
                p_value = value;
                if (p_value < 0) p_value = 0;
                if (p_value > p_limit) p_value = p_limit;
                position.X = start.X + p_value;
            }
        }

        public int Limit
        {
            get { return p_limit; }
            set
            {
                p_limit = value;
                sprBar.scaleV = new Vector2((float)(p_limit + this.image.Width+1), 1.0f);
            }
        }

        public override void Update(TouchLocation touch)
        {
            base.Update(touch);

            Moving = false;

            if (touch.State == TouchLocationState.Moved)
            {
                Rectangle rect = Boundary();
                Point point = new Point((int)touch.Position.X, (int)touch.Position.Y);
                if (rect.Contains(point))
                {
                    Vector2 relative = Vector2.Zero;
                    relative.X = touch.Position.X - position.X;
                    position.X += relative.X;
                    Value = (int)(position.X - start.X);
                    if (position.X < start.X)
                        position.X = start.X;
                    else if (p_value > p_limit)
                        position.X -= relative.X;

                    Moving = true;
                }
            }

        }

        public override void Draw()
        {
            //draw ends
            sprLeftEnd.position = new Vector2(start.X - 16, start.Y);
            sprLeftEnd.color = this.color;
            sprLeftEnd.Draw();
            sprRightEnd.position = new Vector2(start.X + 16 + p_limit, start.Y);
            sprRightEnd.color = this.color;
            sprRightEnd.Draw();

            //draw middle bar
            sprBar.position = new Vector2(start.X - 16, start.Y);
            sprBar.color = this.color;
            sprBar.Draw();


            //draw sliding circle
            base.Draw();

            //draw value text
            Vector2 size = p_font.MeasureString(p_value.ToString());
            p_spriteBatch.DrawString(p_font, p_value.ToString(), position, Color.Black,
                0.0f, new Vector2(size.X/2, size.Y/2), 0.6f, SpriteEffects.None, 1.0f);
        }

        public void SetStartPosition(Vector2 pos)
        {
            position = pos;
            start = pos;
        }

    }

    /*
     * GUI Vertical Slider control
     */
    public class VSlider : Control
    {
        public bool Moving;
        public Vector2 start;
        private int p_value;
        private int p_limit;
        Sprite sprTopEnd, sprBottomEnd, sprBar;

        public VSlider(ContentManager content, SpriteBatch spriteBatch, SpriteFont font)
            : base(content, spriteBatch, font)
        {
            scale = 1.0f;
            start = Vector2.Zero;

            Load("slider_tab");

            sprTopEnd = new Sprite(content, spriteBatch);
            sprTopEnd.Load("vslider_end");
            sprTopEnd.origin = new Vector2(16, 3);

            sprBottomEnd = new Sprite(content, spriteBatch);
            sprBottomEnd.Load("vslider_end");
            sprBottomEnd.origin = new Vector2(16, 0);

            sprBar = new Sprite(content, spriteBatch);
            sprBar.Load("vslider_bar");
            sprBar.origin = new Vector2(16, 0);
            Limit = 100;
        }

        public int Value
        {
            get { return p_value; }
            set
            {
                p_value = value;
                if (p_value < 0) p_value = 0;
                if (p_value > p_limit) p_value = p_limit;
                position.Y = start.Y + p_value;
            }
        }

        public int Limit
        {
            get { return p_limit; }
            set
            {
                p_limit = value;
                sprBar.scaleV = new Vector2(1.0f, (float)(p_limit + this.image.Height + 1));
            }
        }

        public override void Update(TouchLocation touch)
        {
            base.Update(touch);

            Moving = false;

            if (touch.State == TouchLocationState.Moved)
            {
                Rectangle rect = Boundary();
                Point point = new Point((int)touch.Position.X, (int)touch.Position.Y);
                if (rect.Contains(point))
                {
                    Vector2 relative = Vector2.Zero;
                    relative.Y = touch.Position.Y - position.Y;
                    position.Y += relative.Y;
                    Value = (int)(position.Y - start.Y);
                    
                    if (position.Y < start.Y)
                        position.Y = start.Y;
                    else if (p_value > p_limit)
                        position.Y -= relative.Y;

                    Moving = true;
                }
            }

        }

        public override void Draw()
        {
            //draw ends
            sprTopEnd.position = new Vector2(start.X, start.Y - 16);
            sprTopEnd.color = this.color;
            sprTopEnd.Draw();
            sprBottomEnd.position = new Vector2(start.X, start.Y + 16 + p_limit);
            sprBottomEnd.color = this.color;
            sprBottomEnd.Draw();

            //draw middle bar
            sprBar.position = new Vector2(start.X, start.Y - 16);
            sprBar.color = this.color;
            sprBar.Draw();


            //draw sliding circle
            base.Draw();

            //draw value text
            Vector2 size = p_font.MeasureString(p_value.ToString());
            p_spriteBatch.DrawString(p_font, p_value.ToString(), position, 
                Color.Black, 0.0f, new Vector2(size.X / 2, size.Y / 2), 0.6f, 
                SpriteEffects.None, zindex);
        }

        public void SetStartPosition(Vector2 pos)
        {
            position = pos;
            start = pos;
        }

    }

}
