using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;
using GameLibrary;

namespace Black_Hole_Game
{
    public class PlayingModule : IGameModule
    {
        Game1 game;
        SpriteFont font;
        Random rand;
        Sprite background;
        MassiveObject blackHole;
        MassiveObject superCore;
        MassiveObject ship;
        float energy = 100.0f;
        int startTime, lifetime;
        List<MassiveObject> objects;

        public PlayingModule(Game1 game)
        {
            this.game = game;
            rand = new Random();
            startTime = 0;
            lifetime = 4000;
        }

        public void LoadContent(ContentManager content)
        {
            font = content.Load<SpriteFont>("WascoSans");

            background = new Sprite(game.Content, game.spriteBatch);
            background.Load("space");
            background.origin = Vector2.Zero;

            blackHole = new MassiveObject(game.Content, game.spriteBatch);
            blackHole.Load("blackhole");
            blackHole.position = new Vector2(500, 240);
            blackHole.scale = 2.0f;
            blackHole.mass = 40;
            blackHole.color = new Color(255, 100, 100, 200);
            blackHole.velocityAngular = 0.1f;

            superCore = new MassiveObject(game.Content, game.spriteBatch);
            superCore.image = blackHole.image;
            superCore.position = new Vector2(blackHole.position.X, blackHole.position.Y);
            superCore.scale = blackHole.scale * 0.4f;
            superCore.mass = 60;
            superCore.color = new Color(200, 100, 100, 180);
            superCore.velocityAngular = 4.0f;
            superCore.origin = new Vector2(64, 64);

            //create objects list
            objects = new List<MassiveObject>();

            //create player ship
            ship = new MassiveObject(game.Content, game.spriteBatch);
            ship.Load("ship");
            ship.position = new Vector2(200, 240);
            ship.mass = 100f;
            ship.scale = 0.2f;
            ship.rotation = MathHelper.ToRadians(90);

        }

        public void Update(TouchLocation touch, GameTime gameTime)
        {
            int time = gameTime.ElapsedGameTime.Milliseconds;

            blackHole.Update(gameTime);
            blackHole.Rotate();
            superCore.Update(gameTime);
            superCore.Rotate();

            foreach (MassiveObject obj in objects)
            {
                if (!obj.alive) continue;

                obj.Update(gameTime);
                obj.Rotate();
                obj.Animate(time);
                obj.Animate();

                //allow ship to collect energy satellites for bonus energy
                if (obj.scale > 3.0f && obj.name == "satellite")
                {
                    obj.Attract(ship);
                    obj.color = Color.White;

                    if (game.RadialCollision(obj.position, ship.position, obj.size.X, 40))
                    {
                        obj.alive = false;
                        energy += obj.scale;
                    }
                }

                if (!obj.captured)
                {
                    //attract when object is near the black hole
                    if (game.RadialCollision(obj.position, blackHole.position, 10, 500))
                    {
                        obj.Attract(blackHole);
                        obj.Attract(superCore);

                        //is object touching the outer edges of the black hole?
                        if (game.RadialCollision(obj.position, blackHole.position, 10, 120))
                        {
                            obj.color = Color.Red;

                            if (obj.name == "satellite")
                            {
                                obj.scale += 0.1f;
                                energy += 0.5f;
                            }

                            obj.Attract(blackHole);
                            obj.Attract(superCore);

                            //oh no, object is caught by the black hole!
                            if (game.RadialCollision(obj.position, superCore.position, 16, 60))
                            {
                                obj.captured = true;
                                obj.lifetime = 3000;
                                obj.startTime = (int)gameTime.TotalGameTime.TotalMilliseconds;
                                OrbitalMovement anim1 = new OrbitalMovement(blackHole.position,
                                    10 + rand.Next(40), obj.rotation, -0.8f);
                                obj.animations.Add(anim1);
                            }
                        }
                        else
                        {
                            obj.color = Color.White;
                        }
                    }
                }

                //when captured, time runs out
                if (obj.lifetime > 0)
                {
                    if (obj.startTime + obj.lifetime < gameTime.TotalGameTime.TotalMilliseconds)
                        obj.alive = false;
                }

                //see if object has gone too far out of bounds
                if (obj.position.X < -200 || obj.position.X > 1000 ||
                    obj.position.Y < -200 || obj.position.Y > 700)
                    obj.alive = false;

            }


            //update ship
            ship.Update(gameTime);
            ship.Rotate();
            ship.Animate(time);
            if (energy <= 0)
            {
                ship.Attract(blackHole);

                //uh oh, object is caught by the black hole
                if (game.RadialCollision(ship.position, superCore.position, 64, 40))
                {
                    ship.captured = true;
                    ship.lifetime = 3000;
                    ship.startTime = (int)gameTime.TotalGameTime.TotalMilliseconds;
                    OrbitalMovement anim1 = new OrbitalMovement(blackHole.position,
                        10 + rand.Next(40), ship.rotation, -0.8f);
                    ship.animations.Add(anim1);
                }

                //done being squished?
                if (ship.lifetime > 0)
                {
                    if (ship.startTime + ship.lifetime < gameTime.TotalGameTime.TotalMilliseconds)
                        ship.alive = false;
                }

            }
            else
            {
                energy -= 0.05f;
                ship.velocityLinear.X = 0.0f;
            }


            //check user input
            if (touch.State == TouchLocationState.Released)
            {
                if (touch.Position.X > ship.position.X)
                {
                    CreateSatellite();
                }
                else
                {
                    if (touch.Position.Y < 200)
                        ship.velocityAngular = -0.01f;
                    else if (touch.Position.Y > 280)
                        ship.velocityAngular = 0.01f;
                    else
                        ship.velocityAngular = 0;
                }
                
            }

            //time to add another random asteroid?
            if (startTime + lifetime < gameTime.TotalGameTime.TotalMilliseconds)
            {
                startTime = (int)gameTime.TotalGameTime.TotalMilliseconds;
                CreateAsteroid();
            }

            //clean out the dead objects
            foreach (MassiveObject obj in objects)
            {
                if (obj.alive == false)
                {
                    objects.Remove(obj);
                    break;
                }
            }

        }

        public void Draw(GameTime gameTime)
        {
            background.Draw();

            superCore.Draw();
            blackHole.Draw();
            ship.Draw();

            foreach (MassiveObject obj in objects)
            {
                if (obj.alive)
                {
                    obj.Draw();
                }
            }


            string text = "Ship rot " + MathHelper.ToDegrees( ship.rotation).ToString();
            game.spriteBatch.DrawString(font, text, new Vector2(0, 0), Color.White);

            text = "Objects " + objects.Count.ToString();
            game.spriteBatch.DrawString(font, text, new Vector2(0, 20), Color.White);

            text = "Energy " + energy.ToString("N0");
            game.spriteBatch.DrawString(font, text, new Vector2(650, 0), Color.White);
        }


        public void CreateAsteroid()
        {
            MassiveObject obj = new MassiveObject(game.Content, game.spriteBatch);
            obj.Load("asteroid");
            obj.columns = 8;
            obj.totalFrames = 64;
            obj.scale = 0.1f + (float)rand.NextDouble();
            obj.size = new Vector2(60, 60);
            obj.radius = 80;

            //randomly place at top or bottom of screen
            obj.position = new Vector2(rand.Next(100, 800), -100);
            obj.velocityLinear = new Vector2(4.0f, (float)(rand.NextDouble() * 6.0));
            if (rand.Next(2) == 1)
            {
                obj.position.Y = -obj.position.Y;
                obj.velocityLinear.Y = -obj.velocityLinear.Y;
            }

            obj.scale = 0.4f;
            obj.mass = 1;
            obj.velocityAngular = 0.001f;
            obj.lifetime = 0;
            obj.name = "asteroid";
            objects.Add(obj);
        }

        public void CreateSatellite() 
        {
            MassiveObject obj;
            obj = new MassiveObject(game.Content, game.spriteBatch);
            obj.Load("plasma32");
            obj.position = ship.position;
            obj.mass = 1;
            obj.scale = 0.5f;
            obj.lifetime = 0;
            obj.name = "satellite";

            //calculate velocity based on ship's angle
            float accel = 4.0f;
            float angle = ship.rotation - MathHelper.ToRadians(90);
            float x = (float)Math.Cos(angle) * accel;
            float y = (float)Math.Sin(angle) * accel;
            obj.velocityLinear = new Vector2(x,y);

            //load energy to launch
            energy -= 1;

            objects.Add(obj);
        }

    }
}
